from pyhdf.HDF import *
from pyhdf.VS import *
from pyhdf.SD import *


class HDF4VD(dict):
    def __init__(self, vd):
        super().__init__()
        self._vd = vd
        nrecs, intmode, fields, size, name = self._vd.inquire()
        self.nrecs = nrecs
        self.intmode = intmode
        self.fields = fields
        self.size = size
        self.name = name
        for k, v in zip(self.fields, map(list, zip(*self._vd.read(self.nrecs)))):
            self[k] = v

    def __repr__(self):
        keylist = ', '.join([k for k in self])
        return f'{self.__class__.__name__} {self.name} ({keylist})'

    def detach(self):
        self._vd.detach()


class HDF4VS(dict):
    def vrefgen(self):
        index = -1
        while True:
            try:
                index = self._vs.next(index)
                yield index
            except HDF4Error:
                break

    def __init__(self, file):
        super().__init__()
        try:
            self._hdf = HDF(file)
        except HDF4Error as e:
            print(f'{e}: {file}')
            raise LookupError(e.args)
        self._vs = self._hdf.vstart()
        for vref in self.vrefgen():
            obj = HDF4VD(self._vs.attach(vref))
            self[obj.name] = obj

    def __repr__(self):
        keylist = ', '.join([k for k in self])
        return f'{self.__class__.__name__} ({keylist})'

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type:
            print(f'Exception type: {exc_type}')
        for vd in self.values():
            vd.detach()
        self._hdf.close()
